#!/usr/bin/env bash
# automatically generated by auxilium

set -eE

SCRIPT_DIR=$(cd "$(dirname "$0")"; pwd -P)

{%- for line in comp.pre_build %}
docker compose -f "$SCRIPT_DIR/compose.yml" build {{ line }}
{%- endfor %}

{%- for line in comp.pre_script %}
{{ line }}
{%- endfor %}
docker compose -f "$SCRIPT_DIR/{{ comp.compose_filename }}"{% for x in comp.overwrite %} -f "$SCRIPT_DIR/{{ x }}"{% endfor %} ${@:-up}
