#!/usr/bin/env bash
# automatically generated by auxilium

set -eE

if [ "$1" == "-p" ]
then
    export PIP_CONFIG_FILE=devops/pip/pip.conf
elif [ "$1" != "" ]
then
    echo "the only valid option is -p for exposing the pip.conf"
    exit 1
fi

pip install -e "$PWD[test,dev{% if "docs" in aux %},docs{%endif%}]" --config-settings editable_mode=strict
pre-commit install --config=devops/pre-commit/config.yaml
